@@ -53,7 +53,7 @@ sub Run {
     # only show the escalations on ticket overviews
     return ''
         if $Self->{LayoutObject}->{Action}
-        !~ /^AgentTicket(Queue|(Status|Locked|Watch|Responsible)View)/;
+        !~ /^AgentTicket(Queue|Service|(Status|Locked|Watch|Responsible)View)/;
 
     # check result cache
     my $CacheTime = $Param{Config}->{CacheTime} || 40;
@@ -187,7 +187,7 @@ sub Run {
             }
             elsif ( $Ticket{SolutionTimeNotification} ) {
                 $Self->{LayoutObject}->Block(
-                    Name => 'TicketEscalationSolutionTimeOver',
+                    Name => 'TicketEscalationSolutionTimeWillBeOver',
                     Data => \%Ticket,
                 );
                 my $Data = $Self->{LayoutObject}->Output(
