@@ -79,9 +79,15 @@ sub ValueValidate {
         UserID => $Param{UserID}
     );
 
+    my $CheckRegex = 1;
+    if ( defined $Param{NoValidateRegex} && $Param{NoValidateRegex} ) {
+        $CheckRegex = 0;
+    }
+
     if (
         IsArrayRefWithData( $Param{DynamicFieldConfig}->{Config}->{RegExList} )
         && IsStringWithData( $Param{Value} )
+        && $CheckRegex
         )
     {
         # check regular expressions
@@ -481,7 +487,7 @@ sub SearchFieldParameterBuild {
     # Search for a wild card in the value (also for '%').
     # The '%' is needed for the compatibility with the old removed textarea function (bug#12783) and
     #   can be removed with OTRS 6.
-    if ( $Value && ( $Value =~ m{\*} || $Value =~ m{\%} ) ) {
+    if ( $Value && ( $Value =~ m{\*} || $Value =~ m{\%} || $Value =~ m{\|\|} ) ) {
 
         # change oprator
         $Operator = 'Like';
