@@ -1,6 +1,5 @@
 # --
-# Kernel/System/SupportDataCollector/PluginBase.pm - system data collector plugin base class
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -28,6 +27,8 @@ our %Status2Name = (
     $StatusInfo    => 'Information',
 );
 
+our @ObjectDependencies = ();
+
 sub new {
     my ( $Type, %Param ) = @_;
 
@@ -35,14 +36,6 @@ sub new {
     my $Self = {};
     bless( $Self, $Type );
 
-    # check object list for completeness
-    for my $Object (
-        qw( ConfigObject LogObject MainObject DBObject EncodeObject XMLObject EnvironmentObject TimeObject TicketObject)
-        )
-    {
-        $Self->{$Object} = $Param{$Object} || die "Got no $Object!";
-    }
-
     return $Self;
 }
 
@@ -109,6 +102,9 @@ sub _AddResult {
         $Result{Identifier} = Scalar::Util::blessed($Self);
     }
 
+    $Result{ShortIdentifier} = $Result{Identifier};
+    $Result{ShortIdentifier} =~ s{Kernel::System::SupportDataCollector::Plugin(Asynchronous)?::}{}xmsg;
+
     $Result{DisplayPath} //= $Self->GetDisplayPath();
 
     $Self->{Results} //= [];
