@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -12,18 +12,33 @@ use strict;
 use warnings;
 
 use if $^O eq 'MSWin32', "Win32::Console::ANSI";
+use File::stat;
 use Term::ANSIColor;
 
-use Kernel::System::Environment;
+use Kernel::System::ObjectManager;
+
+# UnitTest helper must be loaded to override the builtin time functions!
+use Kernel::System::UnitTest::Helper;
+
+our @ObjectDependencies = (
+    'Kernel::Config',
+    'Kernel::System::DB',
+    'Kernel::System::Encode',
+    'Kernel::System::JSON',
+    'Kernel::System::Log',
+    'Kernel::System::Main',
+    'Kernel::System::SupportDataCollector',
+    'Kernel::System::Time',
+    'Kernel::System::WebUserAgent',
+);
 
 =head1 NAME
 
-Kernel::System::UnitTest - global test interface
+Kernel::System::UnitTest - global unit test interface
 
 =head1 SYNOPSIS
 
-Functions to run existing unit tests, as well as
-functions to define test cases.
+Functions to run existing unit tests, as well as functions to define test cases.
 
 =head1 PUBLIC INTERFACE
 
@@ -33,47 +48,11 @@ functions to define test cases.
 
 =item new()
 
-create unit test object
+create unit test object. Do not use it directly, instead use:
 
-    use Kernel::Config;
-    use Kernel::System::Encode;
-    use Kernel::System::Log;
-    use Kernel::System::Main;
-    use Kernel::System::DB;
-    use Kernel::System::Time;
-    use Kernel::System::UnitTest;
-
-    my $ConfigObject = Kernel::Config->new();
-    my $EncodeObject = Kernel::System::Encode->new(
-        ConfigObject => $ConfigObject,
-    );
-    my $LogObject = Kernel::System::Log->new(
-        ConfigObject => $ConfigObject,
-        EncodeObject => $EncodeObject,
-    );
-    my $MainObject = Kernel::System::Main->new(
-        ConfigObject => $ConfigObject,
-        EncodeObject => $EncodeObject,
-        LogObject    => $LogObject,
-    );
-    my $TimeObject = Kernel::System::Time->new(
-        ConfigObject => $ConfigObject,
-        LogObject    => $LogObject,
-    );
-    my $DBObject = Kernel::System::DB->new(
-        ConfigObject => $ConfigObject,
-        EncodeObject => $EncodeObject,
-        LogObject    => $LogObject,
-        MainObject   => $MainObject,
-    );
-    my $UnitTestObject = Kernel::System::UnitTest->new(
-        EncodeObject => $EncodeObject,
-        ConfigObject => $ConfigObject,
-        LogObject    => $LogObject,
-        MainObject   => $MainObject,
-        DBObject     => $DBObject,
-        TimeObject   => $TimeObject,
-    );
+    use Kernel::System::ObjectManager;
+    local $Kernel::OM = Kernel::System::ObjectManager->new();
+    my $UnitTestObject = $Kernel::OM->Get('Kernel::System::UnitTest');
 
 =cut
 
@@ -86,38 +65,14 @@ sub new {
 
     $Self->{Debug} = $Param{Debug} || 0;
 
-    # check needed objects
-    for (qw(ConfigObject DBObject LogObject TimeObject MainObject EncodeObject)) {
-        if ( $Param{$_} ) {
-            $Self->{$_} = $Param{$_};
-        }
-        else {
-            die "Got no $_!";
-        }
-    }
+    $Self->{ResultData} = undef;
+    $Self->{TestFile}   = '';
 
-    # create additional objects
-    $Self->{EnvironmentObject} = Kernel::System::Environment->new( %{$Self} );
-
-    $Self->{Output} = $Param{Output} || 'ASCII';
-
-    if ( $Self->{Output} eq 'HTML' ) {
-        print "
-<html>
-<head>
-    <title>"
-            . $Self->{ConfigObject}->Get('Product') . " "
-            . $Self->{ConfigObject}->Get('Version')
-            . " - Test Summary</title>
-</head>
-<a name='top'></a>
-<body>
-
-\n";
-    }
-
-    $Self->{XML}     = undef;
-    $Self->{XMLUnit} = '';
+    # Make sure stuff is always flushed to keep it in the right order.
+    *STDOUT->autoflush(1);
+    *STDERR->autoflush(1);
+    $Self->{OriginalSTDOUT} = *STDOUT;
+    $Self->{OriginalSTDOUT}->autoflush(1);
 
     return $Self;
 }
@@ -127,8 +82,14 @@ sub new {
 Run all tests located in scripts/test/*.t and print result to stdout.
 
     $UnitTestObject->Run(
-        Name      => 'JSON:User:Auth',  # optional, control which tests to select
-        Directory => 'Selenium',        # optional, control which tests to select
+        Tests                  => 'JSON:User:Auth',     # optional, execute certain test files only
+        Directory              => 'Selenium',           # optional, execute tests in subdirectory
+        Verbose                => 1,                    # optional (default 0), show result details for all tests, not just failing
+        SubmitURL              => $URL,                 # optional, send results to unit test result server
+        SubmitAuth             => '0abc86125f0fd37baae' # optional authentication string for unit test result server
+        SubmitResultAsExitCode => 1,                    # optional, specify if exit code should not indicate if tests were ok/not ok, but if submission was successful instead
+        JobID                  => 12,                   # optional job ID for unit test submission to server
+        Scenario               => 'OTRS 4 git',         # optional scenario identifier for unit test submission to server
     );
 
 =cut
@@ -136,121 +97,249 @@ Run all tests located in scripts/test/*.t and print result to stdout.
 sub Run {
     my ( $Self, %Param ) = @_;
 
-    my %ResultSummary;
-    my $Home = $Self->{ConfigObject}->Get('Home');
+    $Self->{Verbose} = $Param{Verbose};
 
+    my $Product
+        = $Kernel::OM->Get('Kernel::Config')->Get('Product') . " " . $Kernel::OM->Get('Kernel::Config')->Get('Version');
+
+    my $Home      = $Kernel::OM->Get('Kernel::Config')->Get('Home');
     my $Directory = "$Home/scripts/test";
-
-    # custom subdirectory passed
     if ( $Param{Directory} ) {
         $Directory .= "/$Param{Directory}";
         $Directory =~ s/\.//g;
     }
 
-    my @Files = $Self->{MainObject}->DirectoryRead(
-        Directory => $Directory,
-        Filter    => [ '*.t', '*/*.t', '*/*/*.t', '*/*/*/*.t', '*/*/*/*/*.t', '*/*/*/*/*/*.t' ],
-    );
-
-    my $StartTime = $Self->{TimeObject}->SystemTime();
-    my $Product   = $Param{Product}
-        || $Self->{ConfigObject}->Get('Product') . " " . $Self->{ConfigObject}->Get('Version');
-    my @Names = split( /:/, $Param{Name} || '' );
+    my @TestsToExecute = split( /:/, $Param{Tests} || '' );
 
     $Self->{TestCountOk}    = 0;
     $Self->{TestCountNotOk} = 0;
+
+    my $StartTime = $Kernel::OM->Get('Kernel::System::Time')->SystemTime();
+
+    my @Files = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
+        Directory => $Directory,
+        Filter    => '*.t',
+        Recursive => 1,
+    );
+
+    FILE:
     for my $File (@Files) {
 
         # check if only some tests are requested
-        if (@Names) {
-            my $Use = 0;
-            for my $Name (@Names) {
-                if ( $Name && $File =~ /\/\Q$Name\E\.t$/ ) {
-                    $Use = 1;
-                }
-            }
-            if ( !$Use ) {
-                next;
-            }
+        if ( @TestsToExecute && !grep { $File =~ /\/\Q$_\E\.t$/smx } @TestsToExecute ) {
+            next FILE;
         }
+
         $Self->{TestCount} = 0;
-        my $UnitTestFile = $Self->{MainObject}->FileRead( Location => $File );
+
+        my $UnitTestFile = $Kernel::OM->Get('Kernel::System::Main')->FileRead( Location => $File );
         if ( !$UnitTestFile ) {
             $Self->True( 0, "ERROR: $!: $File" );
             print STDERR "ERROR: $!: $File\n";
+            next FILE;
         }
-        else {
-            $Self->_PrintHeadlineStart($File);
 
-            # create a new scope to be sure to destroy local object of the test files
-            {
+        print "+-------------------------------------------------------------------+\n";
+        print "$File:\n";
+        print "+-------------------------------------------------------------------+\n";
 
-                # HERE the actual tests are run!!!
-                if ( !eval ${$UnitTestFile} ) {    ## no critic
-                    $Self->True( 0, "ERROR: Syntax error in $File: $@" );
-                    print STDERR "ERROR: Syntax error in $File: $@\n";
+        $Self->{TestFile} = $File;
+
+        my $FileStartTime = $Kernel::OM->Get('Kernel::System::Time')->SystemTime();
+
+        # create a new scope to be sure to destroy local object of the test files
+        {
+            # Make sure every UT uses its own clean environment.
+            local $Kernel::OM = Kernel::System::ObjectManager->new(
+                'Kernel::System::Log' => {
+                    LogPrefix => 'OTRS-otrs.UnitTest',
+                },
+            );
+
+            # Provide $Self as 'Kernel::System::UnitTest' for convenience.
+            $Kernel::OM->ObjectInstanceRegister(
+                Package      => 'Kernel::System::UnitTest',
+                Object       => $Self,
+                Dependencies => [],
+            );
+
+            push @{ $Self->{NotOkInfo} }, [$File];
+
+            $Self->{OutputBuffer} = '';
+            local *STDOUT = *STDOUT;
+            local *STDERR = *STDERR;
+            if ( !$Param{Verbose} ) {
+                undef *STDOUT;
+                undef *STDERR;
+                open STDOUT, '>:utf8', \$Self->{OutputBuffer};    ## no critic
+                open STDERR, '>:utf8', \$Self->{OutputBuffer};    ## no critic
+            }
+
+            # HERE the actual tests are run!!!
+            if ( !eval ${$UnitTestFile} ) {                       ## no critic
+                if ($@) {
+                    $Self->True( 0, "ERROR: Error in $File: $@" );
+                }
+                else {
+                    $Self->True( 0, "ERROR: $File did not return a true value." );
                 }
             }
+        }
 
-            $Self->_PrintHeadlineEnd($File);
+        my $FileDuration = $Kernel::OM->Get('Kernel::System::Time')->SystemTime() - $FileStartTime;
+        $Self->{ResultData}->{$File}->{Duration} = $FileDuration;
+
+        print "\n";
+    }
+
+    my $EndTime  = $Kernel::OM->Get('Kernel::System::Time')->SystemTime();
+    my $Duration = $EndTime - $StartTime;
+
+    my $Host = $Kernel::OM->Get('Kernel::Config')->Get('FQDN');
+
+    print "=====================================================================\n";
+    print "$Host ran tests in ${Duration}s";
+    print " for $Product\n";
+
+    if ( $Self->{TestCountNotOk} ) {
+        print "$Self->{TestCountNotOk} tests failed.\n";
+    }
+    else {
+        if ( $Self->{TestCountOk} ) {
+            print "All $Self->{TestCountOk} tests passed.\n";
+        }
+        else {
+            print "No tests executed.\n";
         }
     }
 
-    my $Time = $Self->{TimeObject}->SystemTime() - $StartTime;
-    $ResultSummary{TimeTaken} = $Time;
-    $ResultSummary{Time}      = $Self->{TimeObject}->SystemTime2TimeStamp(
-        SystemTime => $Self->{TimeObject}->SystemTime(),
-    );
-    $ResultSummary{Product} = $Product;
-    $ResultSummary{Host}    = $Self->{ConfigObject}->Get('FQDN');
-    $ResultSummary{Perl}    = sprintf "%vd", $^V;
-    my %OSInfo = $Self->{EnvironmentObject}->OSInfoGet();
-    $ResultSummary{OS}        = $OSInfo{OS};
-    $ResultSummary{Vendor}    = $OSInfo{OSName};
-    $ResultSummary{Database}  = $Self->{DBObject}->Version();
-    $ResultSummary{TestOk}    = $Self->{TestCountOk};
-    $ResultSummary{TestNotOk} = $Self->{TestCountNotOk};
-
-    $Self->_PrintSummary(%ResultSummary);
-    if ( $Self->{Content} ) {
-        print $Self->{Content};
+    if ( $Self->{TestCountNotOk} ) {
+        print " FailedTests:\n";
+        FAILEDFILE:
+        for my $FailedFile ( @{ $Self->{NotOkInfo} || [] } ) {
+            my ( $File, @Tests ) = @{ $FailedFile || [] };
+            next FAILEDFILE if !@Tests;
+            print sprintf "  %s #%s\n", $File, join ", ", @Tests;
+        }
     }
 
-    if ( $Self->{Output} eq 'XML' ) {
-        my $XML = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n";
-        $XML .= "<otrs_test>\n";
-        $XML .= "<Summary>\n";
-        for my $Key ( sort keys %ResultSummary ) {
-            $ResultSummary{$Key} =~ s/&/&amp;/g;
-            $ResultSummary{$Key} =~ s/</&lt;/g;
-            $ResultSummary{$Key} =~ s/>/&gt;/g;
-            $ResultSummary{$Key} =~ s/"/&quot;/g;
-            $XML .= "  <Item Name=\"$Key\">$ResultSummary{$Key}</Item>\n";
-        }
-        $XML .= "</Summary>\n";
-        for my $Key ( sort keys %{ $Self->{XML}->{Test} } ) {
+    if ( $Param{SubmitURL} ) {
 
-            # extract duration time
-            my $Duration = $Self->{Duration}->{$Key};
+        my %SupportData = $Kernel::OM->Get('Kernel::System::SupportDataCollector')->Collect();
+        die "Could not collect SupportData.\n" if !$SupportData{Success};
 
-            $XML .= "<Unit Name=\"$Key\" Duration=\"$Duration\">\n";
+        my %SubmitData = (
+            Auth     => $Param{SubmitAuth} // '',
+            JobID    => $Param{JobID}      // '',
+            Scenario => $Param{Scenario}   // '',
+            Meta     => {
+                StartTime => $StartTime,
+                Duration  => $Duration,
+                TestOk    => $Self->{TestCountOk},
+                TestNotOk => $Self->{TestCountNotOk},
+            },
+            SupportData => $SupportData{Result},
+            Results     => $Self->{ResultData},
+        );
 
-            for my $TestCount ( sort { $a <=> $b } keys %{ $Self->{XML}->{Test}->{$Key} } ) {
-                my $Result  = $Self->{XML}->{Test}->{$Key}->{$TestCount}->{Result};
-                my $Content = $Self->{XML}->{Test}->{$Key}->{$TestCount}->{Name};
-                $Content =~ s/&/&amp;/g;
-                $Content =~ s/</&lt;/g;
-                $Content =~ s/>/&gt;/g;
-                $XML .= qq|  <Test Result="$Result" Count="$TestCount">$Content</Test>\n|;
+        print "=====================================================================\n";
+        print "Sending results to $Param{SubmitURL} ...\n";
+
+        # Flush possible output log files to be able to submit them.
+        *STDOUT->flush();
+        *STDERR->flush();
+
+        # Limit attachment sizes to 2MB in total.
+        my @AttachmentPath  = split( /:/, $Param{AttachmentPath} || '' );
+        my $AttachmentCount = scalar @AttachmentPath;
+        my $AttachmentsSize = 1024 * 1024 * 2;
+
+        ATTACHMENT_PATH:
+        for my $AttachmentPath (@AttachmentPath) {
+            my $FileHandle;
+            my $Content;
+
+            if ( !open $FileHandle, '<:encoding(UTF-8)', $AttachmentPath ) {    ## no-critic
+                print "Could not open file $AttachmentPath, skipping.\n";
+                next ATTACHMENT_PATH;
             }
 
-            $XML .= "</Unit>\n";
-        }
-        $XML .= "</otrs_test>\n";
+            # Read only allocated size of file to try to avoid out of memory error.
+            if ( !read $FileHandle, $Content, $AttachmentsSize / $AttachmentCount ) {    ## no-critic
+                print "Could not read file $AttachmentPath, skipping.\n";
+                close $FileHandle;
+                next ATTACHMENT_PATH;
+            }
 
-        print $XML;
+            my $Stat = stat($AttachmentPath);
+
+            if ( !$Stat ) {
+                print "Cannot stat file $AttachmentPath, skipping.\n";
+                next ATTACHMENT_PATH;
+            }
+
+            # If file size exceeds the limit, include message about shortening at the end.
+            if ( $Stat->size() > $AttachmentsSize / $AttachmentCount ) {
+                $Content .= "\nThis file has been shortened because of size constraint.";
+            }
+
+            close $FileHandle;
+
+            $SubmitData{Attachments}->{$AttachmentPath} = $Content;
+        }
+
+        my $JSONObject = $Kernel::OM->Get('Kernel::System::JSON');
+
+        # Perform web service request and get response.
+        my %Response = $Kernel::OM->Get('Kernel::System::WebUserAgent')->Request(
+            Type => 'POST',
+            URL  => $Param{SubmitURL},
+            Data => {
+                Action      => 'PublicCIMaster',
+                Subaction   => 'TestResults',
+                RequestData => $JSONObject->Encode(
+                    Data => \%SubmitData,
+                ),
+            },
+        );
+
+        if ( $Response{Status} ne '200 OK' ) {
+            print "Submission to server failed (status code '$Response{Status}').\n";
+            return;
+        }
+
+        if ( !$Response{Content} ) {
+            print "Submission to server failed (no response).\n";
+            return;
+        }
+
+        # Convert internal used charset.
+        $Kernel::OM->Get('Kernel::System::Encode')->EncodeInput(
+            $Response{Content},
+        );
+
+        my $ResponseData = $JSONObject->Decode(
+            Data => ${ $Response{Content} },
+        );
+
+        if ( !$ResponseData ) {
+            print "Submission to server failed (invalid response).\n";
+            return;
+        }
+
+        if ( !$ResponseData->{Success} && $ResponseData->{ErrorMessage} ) {
+            print "Submission to server failed (error message '$ResponseData->{ErrorMessage}').\n";
+            return;
+        }
+
+        print "Submission was successful.\n";
+
+        if ( $Param{SubmitResultAsExitCode} ) {
+            return 1;
+        }
     }
-    return 1;
+
+    return $Self->{TestCountNotOk} ? 0 : 1;
 }
 
 =item True()
@@ -277,7 +366,7 @@ sub True {
     my ( $Self, $True, $Name ) = @_;
 
     if ( !$Name ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name! E. g. True(\$A, \'Test Name\')!'
         );
@@ -308,7 +397,7 @@ sub False {
     my ( $Self, $False, $Name ) = @_;
 
     if ( !$Name ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name! E. g. False(\$A, \'Test Name\')!'
         );
@@ -350,7 +439,7 @@ sub Is {
     my ( $Self, $Test, $ShouldBe, $Name ) = @_;
 
     if ( !$Name ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name! E. g. Is(\$A, \$B, \'Test Name\')!'
         );
@@ -393,7 +482,7 @@ sub IsNot {
     my ( $Self, $Test, $ShouldBe, $Name ) = @_;
 
     if ( !$Name ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name! E. g. IsNot(\$A, \$B, \'Test Name\')!'
         );
@@ -449,7 +538,7 @@ sub IsDeeply {
     my ( $Self, $Test, $ShouldBe, $Name ) = @_;
 
     if ( !$Name ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name! E. g. Is(\$A, \$B, \'Test Name\')!'
         );
@@ -479,8 +568,8 @@ sub IsDeeply {
         return 1;
     }
     else {
-        my $ShouldBeDump = $Self->{MainObject}->Dump($ShouldBe);
-        my $TestDump     = $Self->{MainObject}->Dump($Test);
+        my $ShouldBeDump = $Kernel::OM->Get('Kernel::System::Main')->Dump($ShouldBe);
+        my $TestDump     = $Kernel::OM->Get('Kernel::System::Main')->Dump($Test);
         $Self->_Print( 0, "$Name (is '$TestDump' should be '$ShouldBeDump')" );
         return;
     }
@@ -499,7 +588,7 @@ sub IsNotDeeply {
     my ( $Self, $Test, $ShouldBe, $Name ) = @_;
 
     if ( !$Name ) {
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => 'Need Name! E. g. IsNot(\$A, \$B, \'Test Name\')!'
         );
@@ -532,7 +621,7 @@ sub IsNotDeeply {
     else {
 
         #        $Self->_Print( 0, "$Name (matches the expected value)" );
-        my $TestDump = $Self->{MainObject}->Dump($Test);
+        my $TestDump = $Kernel::OM->Get('Kernel::System::Main')->Dump($Test);
         $Self->_Print( 0, "$Name (The structures are equal: '$TestDump')" );
 
         return;
@@ -551,7 +640,7 @@ they are different, undef otherwise.
 Data parameters need to be passed by reference and can be SCALAR,
 ARRAY or HASH.
 
-    my $DataIsDifferent = $SysConfigObject->_DataDiff(
+    my $DataIsDifferent = $UnitTestObject->_DataDiff(
         Data1 => \$Data1,
         Data2 => \$Data2,
     );
@@ -564,7 +653,7 @@ sub _DataDiff {
     # check needed stuff
     for (qw(Data1 Data2)) {
         if ( !defined $Param{$_} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $_!"
             );
@@ -613,10 +702,11 @@ sub _DataDiff {
         return 1 if $#A ne $#B;
 
         # compare array
+        COUNT:
         for my $Count ( 0 .. $#A ) {
 
             # do nothing, it's ok
-            next if !defined $A[$Count] && !defined $B[$Count];
+            next COUNT if !defined $A[$Count] && !defined $B[$Count];
 
             # return diff, because its different
             return 1 if !defined $A[$Count] || !defined $B[$Count];
@@ -627,7 +717,7 @@ sub _DataDiff {
                         Data1 => $A[$Count],
                         Data2 => $B[$Count]
                     );
-                    next;
+                    next COUNT;
                 }
                 return 1;
             }
@@ -641,13 +731,14 @@ sub _DataDiff {
         my %B = %{ $Param{Data2} };
 
         # compare %A with %B and remove it if checked
+        KEY:
         for my $Key ( sort keys %A ) {
 
             # Check if both are undefined
             if ( !defined $A{$Key} && !defined $B{$Key} ) {
                 delete $A{$Key};
                 delete $B{$Key};
-                next;
+                next KEY;
             }
 
             # return diff, because its different
@@ -656,7 +747,7 @@ sub _DataDiff {
             if ( $A{$Key} eq $B{$Key} ) {
                 delete $A{$Key};
                 delete $B{$Key};
-                next;
+                next KEY;
             }
 
             # return if values are different
@@ -667,7 +758,7 @@ sub _DataDiff {
                 );
                 delete $A{$Key};
                 delete $B{$Key};
-                next;
+                next KEY;
             }
             return 1;
         }
@@ -688,138 +779,69 @@ sub _DataDiff {
     return 1;
 }
 
-sub _PrintSummary {
-    my ( $Self, %ResultSummary ) = @_;
-
-    # show result
-    if ( $Self->{Output} eq 'HTML' ) {
-        print "<table width='600' border='1'>\n";
-        if ( $ResultSummary{TestNotOk} ) {
-            print "<tr><td bgcolor='red' colspan='2'>Summary</td></tr>\n";
-        }
-        else {
-            print "<tr><td bgcolor='green' colspan='2'>Summary</td></tr>\n";
-        }
-        print "<tr><td>Product: </td><td>$ResultSummary{Product}</td></tr>\n";
-        print "<tr><td>Test Time:</td><td>$ResultSummary{TimeTaken} s</td></tr>\n";
-        print "<tr><td>Time:     </td><td> $ResultSummary{Time}</td></tr>\n";
-        print "<tr><td>Host:     </td><td>$ResultSummary{Host}</td></tr>\n";
-        print "<tr><td>Perl:     </td><td>$ResultSummary{Perl}</td></tr>\n";
-        print "<tr><td>OS:       </td><td>$ResultSummary{OS}</td></tr>\n";
-        print "<tr><td>Vendor:   </td><td>$ResultSummary{Vendor}</td></tr>\n";
-        print "<tr><td>Database: </td><td>$ResultSummary{Database}</td></tr>\n";
-        print "<tr><td>TestOk:   </td><td>$ResultSummary{TestOk}</td></tr>\n";
-        print "<tr><td>TestNotOk:</td><td>$ResultSummary{TestNotOk}</td></tr>\n";
-        print "</table><br>\n";
-    }
-    elsif ( $Self->{Output} eq 'ASCII' ) {
-        print "=====================================================================\n";
-        print " Product:   $ResultSummary{Product}\n";
-        print " Test Time: $ResultSummary{TimeTaken} s\n";
-        print " Time:      $ResultSummary{Time}\n";
-        print " Host:      $ResultSummary{Host}\n";
-        print " Perl:      $ResultSummary{Perl}\n";
-        print " OS:        $ResultSummary{OS}\n";
-        print " Vendor:    $ResultSummary{Vendor}\n";
-        print " Database:  $ResultSummary{Database}\n";
-        print " TestOk:    $ResultSummary{TestOk}\n";
-        print " TestNotOk: $ResultSummary{TestNotOk}\n";
-        print "=====================================================================\n";
-    }
-    return 1;
-}
-
-sub _PrintHeadlineStart {
-    my ( $Self, $Name ) = @_;
-
-    # set default name
-    $Name ||= '->>No Name!<<-';
-
-    if ( $Self->{Output} eq 'HTML' ) {
-        $Self->{Content} .= "<table width='600' border='1'>\n";
-        $Self->{Content} .= "<tr><td colspan='2'>$Name</td></tr>\n";
-    }
-    elsif ( $Self->{Output} eq 'ASCII' ) {
-        print "+-------------------------------------------------------------------+\n";
-        print "$Name:\n";
-        print "+-------------------------------------------------------------------+\n";
-    }
-
-    $Self->{XMLUnit} = $Name;
-
-    # set duration start time
-    $Self->{DurationStartTime}->{$Name} = $Self->{TimeObject}->SystemTime();
-
-    return 1;
-}
-
-sub _PrintHeadlineEnd {
-    my ( $Self, $Name ) = @_;
-
-    # set default name
-    $Name ||= '->>No Name!<<-';
-
-    if ( $Self->{Output} eq 'HTML' ) {
-        $Self->{Content} .= "</table><br>\n";
-    }
-    elsif ( $Self->{Output} eq 'ASCII' ) {
-    }
-
-    # calculate duration time
-    my $Duration = '';
-    if ( $Self->{DurationStartTime}->{$Name} ) {
-
-        $Duration = $Self->{TimeObject}->SystemTime() - $Self->{DurationStartTime}->{$Name};
-
-        delete $Self->{DurationStartTime}->{$Name};
-    }
-    $Self->{Duration}->{$Name} = $Duration;
-
-    return 1;
-}
-
 sub _Print {
-    my ( $Self, $Test, $Name ) = @_;
-    if ( !$Name ) {
-        $Name = '->>No Name!<<-';
+    my ( $Self, $ResultOk, $Message ) = @_;
+
+    $Message ||= '->>No Name!<<-';
+
+    my $ShortMessage = $Message;
+    if ( length $ShortMessage > 1000 && !$Self->{Verbose} ) {
+        $ShortMessage = substr( $ShortMessage, 0, 1000 ) . "...";
     }
 
+    if ( $Self->{Verbose} || !$ResultOk ) {
+        print { $Self->{OriginalSTDOUT} } $Self->{OutputBuffer};
+    }
+    $Self->{OutputBuffer} = '';
+
     $Self->{TestCount}++;
-    if ($Test) {
+    if ($ResultOk) {
         $Self->{TestCountOk}++;
-        if ( $Self->{Output} eq 'HTML' ) {
-            $Self->{Content}
-                .= "<tr><td width='70' bgcolor='green'>ok $Self->{TestCount}</td><td>$Name</td></tr>\n";
+        if ( $Self->{Verbose} ) {
+            print { $Self->{OriginalSTDOUT} } " ok $Self->{TestCount} - $ShortMessage\n";
         }
-        elsif ( $Self->{Output} eq 'ASCII' ) {
-            print color('green') . " ok" . color('reset') . " $Self->{TestCount} - $Name\n";
+        else {
+            print { $Self->{OriginalSTDOUT} } '.';
         }
-        $Self->{XML}->{Test}->{ $Self->{XMLUnit} }->{ $Self->{TestCount} }->{Result} = 'ok';
-        $Self->{XML}->{Test}->{ $Self->{XMLUnit} }->{ $Self->{TestCount} }->{Name}   = $Name;
+        $Self->{ResultData}->{ $Self->{TestFile} }->{TestOk}++;
         return 1;
     }
     else {
         $Self->{TestCountNotOk}++;
-        if ( $Self->{Output} eq 'HTML' ) {
-            $Self->{Content}
-                .= "<tr><td width='70' bgcolor='red'>not ok $Self->{TestCount}</td><td>$Name</td></tr>\n";
+        if ( !$Self->{Verbose} ) {
+            print { $Self->{OriginalSTDOUT} } "\n";
         }
-        elsif ( $Self->{Output} eq 'ASCII' ) {
-            print color('red') . " not ok" . color('reset') . " $Self->{TestCount} - $Name\n";
+        print { $Self->{OriginalSTDOUT} } " not ok $Self->{TestCount} - $ShortMessage\n";
+        $Self->{ResultData}->{ $Self->{TestFile} }->{TestNotOk}++;
+        $Self->{ResultData}->{ $Self->{TestFile} }->{Results}->{ $Self->{TestCount} }->{Status}  = 'not ok';
+        $Self->{ResultData}->{ $Self->{TestFile} }->{Results}->{ $Self->{TestCount} }->{Message} = $Message;
+
+        my $TestFailureDetails = $Message;
+        $TestFailureDetails =~ s{\(.+\)$}{};
+        if ( length $TestFailureDetails > 200 ) {
+            $TestFailureDetails = substr( $TestFailureDetails, 0, 200 ) . "...";
         }
-        $Self->{XML}->{Test}->{ $Self->{XMLUnit} }->{ $Self->{TestCount} }->{Result} = 'not ok';
-        $Self->{XML}->{Test}->{ $Self->{XMLUnit} }->{ $Self->{TestCount} }->{Name}   = $Name;
+
+        # Store information about failed tests, but only if we are running in a toplevel unit test object
+        #   that is actually processing files, and not in an embedded object that just runs individual tests.
+        if ( ref $Self->{NotOkInfo} eq 'ARRAY' ) {
+            push @{ $Self->{NotOkInfo}->[-1] }, sprintf "%s - %s", $Self->{TestCount},
+                $TestFailureDetails;
+        }
+
         return;
     }
 }
 
-sub DESTROY {
-    my $Self = shift;
+sub AttachSeleniumScreenshot {
+    my ( $Self, %Param ) = @_;
+
+    push @{ $Self->{ResultData}->{ $Self->{TestFile} }->{Results}->{ $Self->{TestCount} }->{Screenshots} },
+        {
+        Filename => $Param{Filename},
+        Content  => $Param{Content},
+        };
 
-    if ( $Self->{Output} eq 'HTML' ) {
-        print "</body>\n";
-        print "</html>\n";
-    }
     return;
 }
 
