@@ -143,6 +143,8 @@ if ( $ENV{nocolors} || $Options =~ m{\A nocolors}msxi ) {
     $NoColors = 1;
 }
 
+my $ExitCode = 0;    # success
+
 # config
 my @NeededModules = (
     {
@@ -372,6 +374,26 @@ my @NeededModules = (
                     zypper => 'perl-IO-Socket-SSL',
                 },
             },
+            {
+                Module    => 'Authen::SASL',
+                Required  => 0,
+                Comment   => 'Required for MD5 authentication mechanisms in IMAP connections.',
+                InstTypes => {
+                    aptget => 'libauthen-sasl-perl',
+                    emerge => 'dev-perl/Authen-SASL',
+                    zypper => 'perl-Authen-SASL',
+                },
+            },
+            {
+                Module    => 'Authen::NTLM',
+                Required  => 0,
+                Comment   => 'Required for NTLM authentication mechanism in IMAP connections.',
+                InstTypes => {
+                    aptget => 'libauthen-ntlm-perl',
+                    emerge => 'dev-perl/Authen-NTLM',
+                    zypper => 'perl-Authen-NTLM',
+                },
+            },
         ],
     },
     {
@@ -659,6 +681,7 @@ sub _Check {
             else {
                 print color('red') . 'FAILED!' . color('reset') . " $ErrorMessage\n";
             }
+            $ExitCode = 1;    # error
         }
         else {
             my $OutputVersion = $Version;
@@ -697,6 +720,7 @@ sub _Check {
         if ($Required) {
             $Required = 'required';
             $Color    = 'red';
+            $ExitCode = 1;            # error
         }
         else {
             $Required = 'optional';
@@ -857,4 +881,4 @@ sub _CheckActiveStatePerl {
     return $ActiveStatePerl ? 1 : 0;
 }
 
-exit 0;
+exit $ExitCode;
