@@ -1,8 +1,5 @@
 // --
-// Core.Debug.js - provides debugging functions
-// Copyright (C) 2001-2011 OTRS AG, http://otrs.org/
-// --
-// $Id: Core.Debug.js,v 1.6 2011/02/17 21:30:59 en Exp $
+// Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 // --
 // This software comes with ABSOLUTELY NO WARRANTY. For details, see
 // the enclosed file COPYING for license information (AGPL). If you
@@ -84,28 +81,6 @@ Core.Debug = (function (TargetNS) {
     };
     /*jslint evil: false */
 
-    /**
-     * @exports TargetNS.BrowserCheck as Core.Debug.BrowserCheck
-     * @function
-     * @description
-     *      Checks, if the used browser is not on the OTRS browser blacklist.
-     * @return true if the used browser is *not* on the black list.
-     */
-    TargetNS.BrowserCheck = function () {
-        var AppropriateBrowser = true,
-            BrowserBlackList = Core.Config.Get('BrowserBlackList');
-        if (typeof BrowserBlackList !== 'undefined') {
-            $.each(BrowserBlackList, function (Key, Value) {
-                if ($.isFunction(Value)) {
-                    if (Value()) {
-                        AppropriateBrowser = false;
-                    }
-                }
-            });
-            return AppropriateBrowser;
-        }
-    };
-
     /**
      * @exports TargetNS.SimulateRTLPage as Core.Debug.SimulateRTLPage
      * @function
